<?php
require_once "Crypt/RSA.php";

$api_id = "[YOUR API KEY HERE]";

//This should be the key file you downloaded from us. No need to transform it.
$apikey_filename = "[YOUR API KEY HERE].key";
$api_user_key =  file_get_contents ( $apikey_filename );
$api_base_url = "https://www.aplos.com/hermes/api/v1/";

//Function for encrypting with RSA
function rsa_encrypt($string, $public_key)
{
    //Create an instance of the RSA cypher and load the key into it
    $cipher = new Crypt_RSA();
    $cipher->loadKey($public_key);
    //Set the encryption mode
    $cipher->setEncryptionMode(CRYPT_RSA_ENCRYPTION_PKCS1);
    //Return the encrypted version
    return base64_encode($cipher->encrypt($string));
}

//Function for decrypting with RSA
function rsa_decrypt($string, $private_key)
{
    //Create an instance of the RSA cypher and load the key into it
    $cipher = new Crypt_RSA();
    $cipher->loadKey($private_key);
    //Set the encryption mode
    $cipher->setEncryptionMode(CRYPT_RSA_ENCRYPTION_PKCS1);
    //Return the decrypted version
    return $cipher->decrypt(base64_decode($string));
}

function get_data($url, $accesstoken) {
  $headers = array();
  $headers[] = "Content-length: 0";
  $headers[] = "Content-type: application/json";
  if ($accesstoken != "") {
    $headers[] = "Authorization: Bearer: " . $accesstoken;
  }

  print "Geting URL: " . $url . "\n";
  foreach($headers as $val) {
    print "With headers: " . $val . "\n";
  }


  $ch = curl_init();
  $timeout = 5;
  curl_setopt($ch, CURLOPT_URL, $url);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
  curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
  if ($action == "POST") {
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $postarray);
  }
  $data = curl_exec($ch);

  if(!curl_errno($ch)){
    return $data;
  }else{
    echo 'Curl error: ' . curl_error($ch);
  }
  curl_close($ch);
}

function post_data($url, $accesstoken, $payload) {
  $headers = array();
  $headers[] = "Content-length: " . strlen($payload);
  $headers[] = "Content-type: application/json";
  if ($accesstoken != "") {
    $headers[] = "Authorization: Bearer: " . $accesstoken;
  }

  print "Posting to URL: " . $url . "\n";
  foreach($headers as $val) {
    print "With headers: " . $val . "\n";
  }

  $ch = curl_init();
  $timeout = 5;
  curl_setopt($ch, CURLOPT_URL, $url);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
  curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
  curl_setopt($ch, CURLOPT_POST, 1);
  curl_setopt($ch, CURLOPT_POSTFIELDS, $payload);

  $data = curl_exec($ch);

  if(!curl_errno($ch)){
    return $data;
  }else{
    echo 'Curl error: ' . curl_error($ch);
  }
  curl_close($ch);
}


function api_error_handling($status_code) {
  // Check our responses
  switch ($status_code) {
    case "200":
      echo "Status: " . $status_code . " : The API let me in!\n";
      break;
    case "401":
      echo "Status: " . $status_code . " : Something is wrong with the auth code. Exiting.\n";
      break;
    case "403":
      echo "Status: " . $status_code . " : Method not allowed. Exiting.\n";
      break;
    case "405":
      echo "Status: " . $status_code . " : Unprocessable Entity. Exiting.\n";
      break;
    case "422":
      echo "Status: " . $status_code . " : Problem with the request. Exiting.\n";
      break;
    }
  return(1);
}

function api_auth($api_base_url, $api_id, $api_user_key) {
  # This should return an authorized token so we can do other, more exciting things

  # Actual request goes here.
  $r = get_data($api_base_url . "auth/" . $api_id, "");
  $rd = json_decode($r);
  api_error_handling($rd->status);
  $api_token_encrypted = $rd->data->token;
  $api_token_encrypted_expires = $rd->data->expires;
  $api_access_token = rsa_decrypt(($api_token_encrypted), $api_user_key);
  print "The API Token expires: " . $api_token_encrypted_expires ."\n";

  return($api_access_token);
}

function api_contacts_get ($api_base_url, $api_id, $api_access_token) {
  # This should print a contact list from Aplos.
  # Lets show what we're doing.

  # Actual request goes here.
  $r = get_data($api_base_url . "contacts", $api_access_token);
  $rd = json_decode($r);
  api_error_handling($rd->status);

  //What did we get?
  // var_dump($r);
  return ($r);
}

function api_accounts_get ($api_base_url, $api_id, $api_access_token) {
  # This should print an account list from Aplos.
  # Lets show what we're doing.

  # Actual request goes here.
  $r = get_data($api_base_url . "accounts", $api_access_token);
  $rd = json_decode($r);
  api_error_handling($rd->status);

  //What did we get?
  var_dump($r);
  return ($r);
}

function api_transactions_get ($api_base_url, $api_id, $api_access_token) {
  # This should print a transaction list from Aplos.
  # Lets show what we're doing.

  # Actual request goes here.
  $r = get_data($api_base_url . "transactions", $api_access_token);
  $rd = json_decode($r);
  api_error_handling($rd->status);

  //What did we get?
  // var_dump($r);
  return ($r);
}

function api_transactions_post ($api_base_url, $api_id, $api_access_token) {
  # This should post a transaction to Aplos.
  # Of course this assumes valid contact, account, fund.
  $payload = '{"note": "Insert a transaction", "date": "2015-12-31", "contact": { "companyname": "Aplos", "type": "company" }, "lines": [ { "amount": 123.45, "account": { "account_number": 1010 }, "fund": { "id": 647 } }, { "amount": -123.45, "account": { "account_number": 4002 }, "fund": { "id": 647 } } ] }';

  # Actual request goes here.
  $r = post_data($api_base_url . "transactions", $api_access_token, $payload);
  $rd = json_decode($r);
  api_error_handling($rd->status);

  //What did we get?
  var_dump($r);
  return ($r);
}

print "Starting ::\n";
$api_access_token = api_auth($api_base_url, $api_id, $api_user_key);

print "Fetching things ::\n";
//api_contacts_get($api_base_url, $api_id, $api_access_token);
api_accounts_get($api_base_url, $api_id, $api_access_token);
//api_transactions_get($api_base_url, $api_id, $api_access_token);

//print "Pushing a transaction ::\n";
//api_transactions_post($api_base_url, $api_id, $api_access_token);

?>
